(* $Id: menu_configure.ml 295 2004-03-12 18:08:58Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann.
 *
 * This file is part of godi-tools.
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

open Godi_types
open Printf


let help_str =
  "This menu allows you to configure a selected package.\n" ^
  "\n" ^
  "Configuration means that 'make' variables must be set in order to give\n" ^
  "GODI hints where to find system libraries, or to enable special functions\n" ^
  "at compile time. These variables can be edited here; they are later written\n" ^
  "into the file godi.conf.\n" ^
  "\n" ^
  "Note that there are now special configuration packages 'conf-<name>' that\n" ^
  "configure the location of system libraries. These packages have some built-in\n" ^
  "intelligence to auto-configure themselves, but sometimes it is necessary to\n" ^
  "give them hints by setting variables. So if the compilation of a 'conf'\n" ^
  "package fails, you can try to set the variables." ^
  "\n" ^
  "You can test a certain configuration (whether it has options or not) by\n" ^
  "pressing 't'. This invokes the configure script of the package. Note\n" ^
  "that this works only if all required packages are already installed."
;;


let lf_re = Str.regexp "\n";;


let display (pg : Miniui.page) pkg =
  let p = source_pkg pkg in  (* we can only configure source pkgs *)

  let accept_option name value =
    let opts = p # values_of_user_build_options in
    let opts' =
      List.map
	(fun (n,v) ->
	   if n = name then
	     (n,value)
	   else
	     (n,v))
	opts in
    p # set_user_build_options opts'
  in

  let rec show() =
    pg # clear();

    Menu_aux.add_compact_head ~lmargin:6 pg ">>> Configure Package <<<";

    pg # put_line (sprintf "Package:           %s" p#name);

    if p#build_options <> [] then (
      pg # put_line "";
      pg # put_line "Options:";
      let rev_menu = ref [] in
      let k = ref 1 in
      List.iter
	(fun (oname,oval) ->
	   rev_menu := `Item ( [ sprintf "[%2d]" !k;
				 oname;
				 oval
			       ],
			       !k,
			       (fun () ->
				  pg # activate_text_entry
				    ~prompt:(oname ^ " = ")
				    ~value:oval
				    ~callback_ok:(
				      fun v ->
					accept_option oname v;
					show())
				    ()
			       )
			     ) :: !rev_menu;
	   incr k
	)
	p#values_of_user_build_options;

      Menu_aux.add_menu
	pg
	[ `Left 4; `Delim " "; `Right 25; `Delim " = "; `Left 40 ]
	( List.rev !rev_menu );
    );

    if p#build_message <> "" then (
      pg # put_line "";
      pg # put_line "Build message:";
      let lines = Str.split lf_re p#build_message in
      List.iter pg#put_line lines;
    );

    ignore(pg # add_foot_button
	     ~text:"test configuration" ~underline:0
	     ~callback:(Menu_testconf.display pg#new_page pkg)
	     (`Keypress 't'));

    ignore(pg # add_foot_button ~text:"help" ~underline:0
             ~callback:(Menu_help.display pg#new_page help_str)
             (`Keypress 'h'));

    ignore(pg # add_foot_button ~text:"exit menu" ~underline:1
             ~callback:pg#quit (`Keypress 'x'));
  in

  ( match pkg with
	`Source_pkg sp ->
	  sp # set_config_dialog_shown true
      | _ ->
	  ()
  );

  try
    show();
    pg # loop()
  with
      User_error msg ->
	Menu_aux.error_page pg msg
;;

